import os
import sys


def open23(name, mode='r'):
    if sys.version_info < (3, 0, 0):
        return open(name, mode)
    else:
        return open(name, mode, encoding='latin-1')


def map23(foo, iterable):
    if sys.version_info < (3, 0, 0):
        return map(foo, iterable)
    else:
        return list(map(foo, iterable))


def file_in_dir(f, d):
    if d:
        return os.path.abspath(f).startswith(os.path.abspath(d) + os.path.sep)
    else:
        return True


def find_unquoted_string(string, line, quotes='\'"'):
    skip = 0
    quote = None
    while 1:
        idx = line.find(string, skip)
        if idx < 0:
            return idx

        escaped = False
        for c in line[skip:idx]:
            if escaped:
                escaped = False
            elif c in quotes:
                if quote is None:
                    quote = c
                elif quote == c:
                    quote = None
            elif c == '\\' and quote:
                escaped = True

        if quote:
            skip = idx + len(string)
        else:
            return idx


class IncludeFinder:
    def __init__(self, include_order=None, include_dirs=None):
        self.include_order = include_order
        self.include_dirs = include_dirs

    def find(self, filename, root_includer=None, current_includer=None):
        if os.path.isabs(filename) and os.path.isfile(filename):
            return filename
        elif self.include_order:
            for inc_type in self.include_order:
                if inc_type == 'cwd' and os.path.isfile(filename):
                    return filename
                elif inc_type == 'src' and root_includer:
                    candidate = os.path.join(os.path.dirname(root_includer),
                                             filename)
                    if os.path.isfile(candidate):
                        return candidate
                elif inc_type == 'inc' and current_includer:
                    candidate = os.path.join(os.path.dirname(current_includer),
                                             filename)
                    if os.path.isfile(candidate):
                        return candidate
                elif inc_type == 'flg' and self.include_dirs:
                    for d in self.include_dirs:
                        candidate = os.path.join(d, filename)
                        if os.path.isfile(candidate):
                            return candidate
        return None


class StreamStack:
    def __init__(self):
        # Stack of file-like objects (i.e. objects implementing methods
        # readline, close, and a property name:
        self._stream_stack = []
        self._close_stack = []

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, exc_traceback):
        self.clear()

    @property
    def root_name(self):
        return self._stream_stack[0].name if self._stream_stack else None

    @property
    def current_name(self):
        return self._stream_stack[-1].name if self._stream_stack else None

    def add(self, stream, close=True):
        self._stream_stack.append(stream)
        self._close_stack.append(close)

    def clear(self):
        for stream, close in zip(self._stream_stack, self._close_stack):
            if close:
                stream.close()
        self._stream_stack *= 0
        self._close_stack *= 0

    def readline(self):
        while self._stream_stack:
            line = self._stream_stack[-1].readline()
            if line:
                return line
            else:
                stream = self._stream_stack.pop()
                if self._close_stack.pop():
                    stream.close()
        return ''


class StdStreamWrapper:
    def __init__(self, stream, name=None):
        self._stream = stream
        self.name = stream.name if name is None else name

    def readline(self):
        return self._stream.readline()

    def writelines(self, lines):
        return self._stream.writelines(lines)

    def close(self):
        pass


class DummyParser:
    def __init__(self):
        pass

    @staticmethod
    def parse(stream):
        while 1:
            line = stream.readline()
            if not line:
                break



#!/usr/bin/env python
import os
import sys

from depgen import open23, map23, StdStreamWrapper, DummyParser

try:
    import argparse
except ImportError:
    import _argparse as argparse


def parse_args():
    class ArgumentParser(argparse.ArgumentParser):
        def convert_arg_line_to_args(self, arg_line):
            try:
                # Drop everything after the first occurrence of #:
                arg_line = arg_line[:arg_line.index('#')]
            except ValueError:
                pass

            result = []
            # Do not regard consecutive whitespaces as a single separator:
            for arg in arg_line.split(' '):
                if arg:
                    result.append(arg)
                elif result:
                    # The previous argument has a significant whitespace:
                    result[-1] += ' '
            return result

    parser = ArgumentParser(
        fromfile_prefix_chars='@',
        description='Generates OUTPUT makefile containing dependency rules '
                    'for the INPUT source file. Recognizes preprocessor '
                    '`#include`, `#if` and associated directives as well as '
                    'Fortran `INCLUDE`, `USE` and `MODULE` statements.')

    def comma_splitter(s):
        return list(filter(None, s.lower().split(',')))

    def path_splitter(s):
        return list(filter(None, s.split(':')))

    def default_obj_name(src_name):
        if src_name:
            src_no_ext_basename = os.path.splitext(
                os.path.basename(src_name))[0]
            if src_no_ext_basename:
                return src_no_ext_basename + '.o'
        return ''

    parser.add_argument(
        '--input', '-i', metavar='INPUT', nargs='+',
        help='input source file; if not specified, the program reads from the '
             'standard input stream')
    parser.add_argument(
        '--output', '-o', metavar='OUTPUT', nargs='+',
        help='output makefile with generated dependency rules; if not '
             'specified, the program writes to the standard output stream')
    parser.add_argument(
        '--debug', '-d', action='store_true',
        help='dump debug information to OUTPUT (commented with `#`)')
    parser.add_argument(
        '--src-name', metavar='SRC_NAME', nargs='+',
        help='name of the source file, the prerequisite of the corresponding '
             'compilation rule as it will appear in the OUTPUT; normally (and '
             'by default) equals to the INPUT or to an empty string when the '
             'latter is set to the standard input stream')
    parser.add_argument(
        '--obj-name', metavar='OBJ_NAME', nargs='+',
        help='name of the object file, the target of the corresponding '
             'compilation rule as it will appear in the OUTPUT; normally '
             'equals to the path to the object file that is supposed to be '
             'generated as a result of compilation (default: SRC_NAME '
             'without the directory-part and the file extension replaced '
             'with `.o`)')
    parser.add_argument(
        '--dep-name', metavar='DEP_NAME', nargs='+',
        help='name of the generated makefile, the additional target of the '
             'corresponding compilation rule (for automatic dependency '
             'generation) as it will appear in the OUTPUT; normally (and by '
             'default) equals to the OUTPUT or to an empty string when the '
             'latter is set to the standard output stream)')
    parser.add_argument(
        '--src-roots', metavar='SRC_ROOTS',
        type=path_splitter,
        help='colon-separated list of paths to directories; if specified and '
             'not empty, dependencies on files that do not reside in one of '
             'the specified directories will be ignored; applies only to '
             'files included using the preprocessor `#include` directive or '
             'the Fortran `INCLUDE` statement')
    parser.add_argument(
        '--lc-enable', action='store_true',
        help='enable recognition of the preprocessor line control directives '
             'and generation of additional dependencies based on the detected '
             'filenames')
    parser.add_argument(
        'flags', metavar='-- [$CPPFLAGS | $FCFLAGS]', nargs='?',
        default=argparse.SUPPRESS,
        help='actual flags to be used in compilation, i.e. $(CPPFLAGS) or '
             '$(FCFLAGS), must be given at the end of the command line '
             'following the double dash separator (--); the program searches '
             'these flags for (possibly multiple instances of) PP_INC_FLAG, '
             'PP_MACRO_FLAG, FC_INC_FLAG and FC_MOD_DIR_FLAG; any values '
             'found are used in the dependency generation (in the case of '
             'FC_MOD_DIR_FLAG, only the last value found is used)')

    pp_arg_group = parser.add_argument_group('preprocessor arguments')
    pp_arg_group.add_argument(
        '--pp-enable', action='store_true',
        help='enable the preprocessing stage; if disabled (default), all '
             'arguments of this argument group are ignored')
    pp_arg_group.add_argument(
        '--pp-eval-expr', action='store_true',
        help='enable evaluation of expressions that appear in preprocessor '
             'directives `#if` and `#elif` (does not apply to `#ifdef` and '
             '`#ifndef`, which are always evaluated); if disabled (default) '
             'or evaluation fails, both branches of the directives are '
             'included by the preprocessing stage')
    pp_arg_group.add_argument(
        '--pp-inc-sys', action='store_true',
        help='enable recognition of dependencies specified with the '
             'angle-bracket form of the preprocessor `#include` directive '
             '(i.e. `#include <filename>`); the constraint set by SRC_ROOTS '
             'applies')
    pp_arg_group.add_argument(
        '--pp-inc-order', default='inc,flg', metavar='ORDER_LIST',
        type=comma_splitter,
        help='directory search order of files included using the quoted form '
             'of the preprocessor `#include` directive (i.e. `#include '
             '"filename"`) ; ORDER_LIST is an ordered comma-separated list of '
             'keywords, the corresponding search paths of which are to be '
             'searched in the given order. The recognized keywords are: `cwd` '
             '(for the current working directory), `flg` (for the directories '
             'specified with PP_INC_FLAG compiler flag), `src` (for the '
             'directory containing the INPUT source file), and `inc` (for the '
             'directory containing the file with the `#include` directive). '
             'Default: `%(default)s`.')
    pp_arg_group.add_argument(
        '--pp-inc-sys-order', default='flg', metavar='ORDER_LIST',
        type=comma_splitter,
        help='equivalent to the `--pp-inc-order` argument, only for the '
             'angle-bracket form of the preprocessor `#include` directive '
             '(i.e. `#include <filename>`, default: `%(default)s`)')

    pp_arg_group.add_argument(
        '--pp-macro-flag', metavar='PP_MACRO_FLAG', default='-D',
        help='preprocessor flag used for macro definition; only flags '
             'that start with a single dash (-) and have no more than one '
             'trailing whitespace are supported (default: `%(default)s`)')
    pp_arg_group.add_argument(
        '--pp-inc-flag', metavar='PP_INC_FLAG', default='-I',
        help='preprocessor flag used for setting search paths for the '
             '`#include` directive; only flags that start with a single dash '
             '(-) and have no more than one trailing whitespace are supported '
             '(default: `%(default)s`)')

    fc_arg_group = parser.add_argument_group('Fortran arguments')
    fc_arg_group.add_argument(
        '--fc-enable', action='store_true',
        help='enable recognition of Fortran dependencies specified with '
             '`INCLUDE`, `USE` and `MODULE` statements; if disabled (default),'
             'all arguments of this argument group are ignored')
    fc_arg_group.add_argument(
        '--fc-mod-ext', default='mod',
        help='filename extension (without leading dot) of compiler-generated '
             'Fortran module files (default: `%(default)s`)')
    fc_arg_group.add_argument(
        '--fc-mod-upper', choices=['yes', 'no'], default='no',
        help='whether Fortran compiler-generated module files have uppercase '
             'names (default: `%(default)s`)')
    fc_arg_group.add_argument(
        '--fc-inc-order', default='src,flg', metavar='ORDER_LIST',
        type=comma_splitter,
        help='equivalent to the `--pp-inc-order` argument, only for the '
             'Fortran `INCLUDE` statement and FC_INC_FLAG (default: '
             '`%(default)s`)')
    fc_intrisic_mods_default = ('iso_c_binding,iso_fortran_env,ieee_exceptions,'
                                'ieee_arithmetic,ieee_features,omp_lib,'
                                'omp_lib_kinds,openacc')
    fc_arg_group.add_argument(
        '--fc-intrinsic-mods', metavar='INTRINSIC_MODS_LIST',
        type=comma_splitter,
        action='append',
        help='comma-separated list of Fortran intrinsic modules. Fortran '
             'modules that are explicitly specified as intrinsic in the '
             'source file (i.e. `USE, INTRINSIC :: MODULENAME`) are ignored '
             'regardless of whether they are mentioned on the '
             'INTRINSIC_MODS_LIST. Fortran modules that are mentioned on the '
             'INTRINSIC_MODS_LIST are ignored only when their nature is not '
             'specified in the source file at all (i.e. `USE :: MODULENAME`). '
             'Fortran modules that need to be ignored unconditionally must '
             'be put on the EXTERNAL_MODS_LIST (see `--fc-external-mods`). '
             'Default: `{0}`.'.format(fc_intrisic_mods_default))
    fc_arg_group.add_argument(
        '--fc-external-mods', metavar='EXTERNAL_MODS_LIST',
        type=comma_splitter,
        action='append',
        help='comma-separated list of external (to the project) Fortran '
             'modules that need to be unconditionally ignored when generating '
             'dependency rules (see also `--fc-intrinsic-mods`)')
    fc_arg_group.add_argument(
        '--fc-mod-dir-flag', metavar='FC_MOD_DIR_FLAG', default='-J',
        help='Fortran compiler flag used to specify the directory where '
             'module files are saved; only flags that start with a single '
             'dash (-) and have no more than one trailing whitespace are '
             'supported (default: `%(default)s`)')
    fc_arg_group.add_argument(
        '--fc-inc-flag', metavar='FC_INC_FLAG', default='-I',
        help='preprocessor flag used for setting search paths for the '
             'Fortran `INCLUDE` statement; only flags that start with a '
             'single dash (-) and have no more than one trailing whitespace '
             'are supported (default: `%(default)s`)')

    unknown = []
    try:
        sep_idx = sys.argv.index('--')
        args = parser.parse_args(sys.argv[1:sep_idx])
        unknown = sys.argv[sep_idx + 1:]
    except ValueError:
        args = parser.parse_args()

    if not args.input:
        args.input = [None]

    if not args.src_name:
        args.src_name = args.input
    elif len(args.src_name) != len(args.input):
        parser.error('number of SRC_NAME values is not equal to the number of '
                     'INPUT values')

    if not args.obj_name:
        args.obj_name = map23(default_obj_name, args.src_name)
    elif len(args.obj_name) != len(args.input):
        parser.error('number of OBJ_NAME values is not equal to the number of '
                     'INPUT values')

    if not args.output:
        args.output = [None] * len(args.input)
    elif len(args.output) != len(args.input):
        parser.error('number of OUTPUT values is not equal to the number of '
                     'INPUT values')

    if not args.dep_name:
        args.dep_name = args.output
    elif len(args.dep_name) != len(args.input):
        parser.error('number of DEP_NAME values is not equal to the number of '
                     'INPUT values')

    compiler_arg_dests = dict()

    if args.pp_enable:
        compiler_arg_dests.update(pp_inc_dirs=args.pp_inc_flag,
                                  pp_macros=args.pp_macro_flag)

    if args.fc_enable:
        compiler_arg_dests.update(fc_inc_dirs=args.fc_inc_flag,
                                  fc_mod_dir=args.fc_mod_dir_flag)

    if compiler_arg_dests:
        compiler_args = dict()
        for dest, flag in compiler_arg_dests.items():
            if not flag.startswith('-') or flag.endswith('  '):
                parser.error('unsupported compiler/preprocessor flag ' + flag)
            # Several dests might share the same flag and we want them to share
            # the same list of values in this the case:
            val_list = compiler_args.get(flag, None)
            if val_list is None:
                val_list = []
                compiler_args[flag] = val_list
            setattr(args, dest, val_list)

        appended_val_lists = []
        for arg in unknown:
            if arg.startswith('-'):
                appended_val_lists *= 0
                arg_ws = arg + ' '
                for flag, val_list in compiler_args.items():
                    if flag == arg or flag == arg_ws:
                        # If the current argument equals to a flag, which might
                        # have a significant trailing whitespace, the next
                        # argument on the command line is the flag's value:
                        appended_val_lists.append(val_list)
                    elif arg.startswith(flag):
                        # If the current argument starts with a flag that does
                        # not have a trailing whitespace, the suffix of the
                        # argument is the flag's value:
                        val_list.append(arg[len(flag):])
            elif appended_val_lists:
                for val_list in appended_val_lists:
                    val_list.append(arg)
                appended_val_lists *= 0

    if args.pp_enable and args.pp_macros:
        import re
        predefined_macros = dict()
        for m in args.pp_macros:
            match = re.match(r'^=*([a-zA-Z_]\w*)(\(.*\))?(?:=(.+))?$', m)
            if match:
                name = match.group(1)
                if name != 'defined':
                    body = match.group(3) if match.group(3) else '1'
                    predefined_macros[name] = (match.group(2), body)
        args.pp_macros = predefined_macros

    if args.fc_enable:
        args.fc_mod_upper = (args.fc_mod_upper == 'yes')
        args.fc_mod_dir = args.fc_mod_dir[-1] if args.fc_mod_dir else None

        if args.fc_intrinsic_mods:
            args.fc_intrinsic_mods = [m for sublist in args.fc_intrinsic_mods
                                      for m in sublist]
        else:
            args.fc_intrinsic_mods = comma_splitter(fc_intrisic_mods_default)

        if args.fc_external_mods:
            args.fc_external_mods = [m for sublist in args.fc_external_mods
                                     for m in sublist]

    return args


def main():
    args = parse_args()

    lc_files = set()

    def lc_callback(filename):
        lc_files.add(filename)

    included_files = set()

    def include_callback(filename):
        included_files.add(filename)

    provided_modules = set()

    def module_callback(module):
        provided_modules.add(module)

    required_modules = set()

    def use_module_callback(module):
        required_modules.add(module)

    lc_debug_info = None
    pp_debug_info = None
    ftn_debug_info = None

    parser = None
    if args.fc_enable:
        from depgen.fortran_parser import FortranParser
        ftn = FortranParser(include_order=args.fc_inc_order,
                            include_dirs=args.fc_inc_dirs,
                            include_roots=args.src_roots,
                            intrinsic_mods=args.fc_intrinsic_mods,
                            external_mods=args.fc_external_mods)

        ftn.include_callback = include_callback
        ftn.module_callback = module_callback
        ftn.use_module_callback = use_module_callback

        def debug_callback(line, msg):
            ftn_debug_info.append('#  `%s`:\t%s\n' % (line[:-1], msg))

        if args.debug:
            ftn_debug_info = ['#\n# Fortran parser:\n']
            ftn.debug_callback = debug_callback

        parser = ftn

    elif args.pp_enable or args.lc_enable:
        parser = DummyParser()

    for inp, out, src_name, obj_name, dep_name in zip(args.input,
                                                      args.output,
                                                      args.src_name,
                                                      args.obj_name,
                                                      args.dep_name):
        in_stream = StdStreamWrapper(sys.stdin, '') \
            if inp is None else open23(inp)

        if args.lc_enable:
            from depgen.line_control import LCProcessor
            lc = LCProcessor(in_stream,
                             include_roots=args.src_roots)
            lc.lc_callback = lc_callback

            def debug_callback(line, msg):
                lc_debug_info.append('#  `%s`:\t%s\n' % (line[:-1], msg))

            if args.debug:
                lc_debug_info = ['#\n# Line control processor:\n']
                lc.debug_callback = debug_callback

            in_stream = lc

        if args.pp_enable:
            from depgen.preprocessor import Preprocessor
            pp = Preprocessor(in_stream,
                              include_order=args.pp_inc_order,
                              include_sys_order=args.pp_inc_sys_order,
                              include_dirs=args.pp_inc_dirs,
                              include_roots=args.src_roots,
                              try_eval_expr=args.pp_eval_expr,
                              inc_sys=args.pp_inc_sys,
                              predefined_macros=args.pp_macros)

            pp.include_callback = include_callback

            def debug_callback(line, msg):
                pp_debug_info.append('#  `%s`:\t%s\n' % (line[:-1], msg))

            if args.debug:
                pp_debug_info = ['#\n# Preprocessor:\n']
                pp.debug_callback = debug_callback

            in_stream = pp

        if parser:
            parser.parse(in_stream)
            in_stream.close()

        out_lines = gen_lc_deps(src_name, lc_files)
        lc_files.clear()

        out_lines.extend(gen_include_deps(src_name, obj_name, dep_name,
                                          included_files))
        included_files.clear()

        if provided_modules or required_modules:
            out_lines.extend(gen_module_deps(obj_name, provided_modules,
                                             required_modules,
                                             args.fc_mod_dir,
                                             args.fc_mod_upper,
                                             args.fc_mod_ext))
        provided_modules.clear()
        required_modules.clear()

        if args.debug:
            out_lines.extend([
                '\n# Python version: ', sys.version.replace('\n', ' '),
                '\n#\n',
                '# Command:\n',
                '#  ', ' '.join(sys.argv), '\n#\n',
                '# Parsed arguments:\n#  ',
                '\n#  '.join(
                    [k + '=' + str(v) for k, v in vars(args).items()]), '\n'])
            if lc_debug_info is not None:
                out_lines.extend(lc_debug_info)
            if pp_debug_info is not None:
                out_lines.extend(pp_debug_info)
            if ftn_debug_info is not None:
                out_lines.extend(ftn_debug_info)
            out_lines.append('\n')

        out_stream = StdStreamWrapper(sys.stdout) \
            if out is None else open23(out, 'w')
        out_stream.writelines(out_lines)
        out_stream.close()


def gen_lc_deps(src_name, lc_files):
    result = []
    if src_name and lc_files:
        result.append('%s: %s\n' % (src_name, ' '.join(lc_files)))
    return result


def gen_include_deps(src_name, obj_name, dep_name,
                     included_files):
    result = []
    targets = ' '.join(filter(None, (obj_name, dep_name)))
    if targets:
        prereqs = ' '.join(filter(None, [src_name] + list(included_files)))
        if prereqs:
            result.append('%s: %s\n' % (targets, prereqs))
    return result


def gen_module_deps(obj_name,
                    provided_modules, required_modules,
                    mod_dir, mod_upper, mod_ext):
    result = []
    if obj_name:
        if provided_modules:
            targets = ' '.join(modulenames_to_filenames(
                provided_modules, mod_dir, mod_upper, mod_ext))
            result.append('%s: %s\n' % (targets, obj_name))

        # Do not depend on the modules that are provided in the same file:
        required_modules -= provided_modules
        if required_modules:
            prereqs = ' '.join(modulenames_to_filenames(
                required_modules, mod_dir, mod_upper, mod_ext))
            result.append('%s: %s\n' % (obj_name, prereqs))
    return result


def modulenames_to_filenames(modules, directory, upprecase, extension):
    result = modules
    if upprecase:
        result = map(lambda s: s.upper(), result)
    if directory:
        result = map(lambda s: os.path.join(directory, s), result)
    if extension:
        result = map(lambda s: '%s.%s' % (s, extension), result)
    return result


if __name__ == "__main__":
    main()



"""
Filename:     io.py
Author:       Shannon Mason, shannon.mason@ecmwf.int
Description:  I/O functions for loading IFS data
"""

#For loading and handling netCDF data
import xarray as xr

def load_inputs(input_srcfile):
    """
    Load ecRAD input files.
    """

    with xr.open_dataset(input_srcfile) as input_dset:
        if 'lat' in input_dset:
            input_dset = input_dset.rename({'lat':'latitude', 'lon':'longitude'})

        input_dset['pressure_fl'] = xr.DataArray(
                input_dset.pressure_hl[:,:-1] + 0.5*input_dset.pressure_hl.diff('half_level'),
                coords={'column':input_dset.column, 'level':input_dset.level},
                dims=['column', 'level']
            )

        #input_dset['p'] = input_dset.pressure_fl.median('column')
        #input_dset['phl'] = input_dset.pressure_hl.median('column')

        if 'aerosol_mmr' in input_dset.data_vars:
            input_dset['sea_salt'] = input_dset.aerosol_mmr.sel(
                    aerosol_type=slice(0,2)).sum('aerosol_type')
            input_dset['dust'] = input_dset.aerosol_mmr.sel(
                    aerosol_type=slice(3,5)).sum('aerosol_type')
            input_dset['organics'] = input_dset.aerosol_mmr.sel(
                    aerosol_type=slice(6,7)).sum('aerosol_type')
            input_dset['black_carbon'] = input_dset.aerosol_mmr.sel(
                    aerosol_type=slice(8,9)).sum('aerosol_type')
            input_dset['sulphate'] = input_dset.aerosol_mmr.sel(aerosol_type=10)

        input_dset = input_dset.set_coords(['latitude', 'longitude']) #, 'p', 'phl'])

        return input_dset.swap_dims({'column':'latitude'})# 'half_level':'phl', 'level':'p'})


def load_ecRAD(output_srcfile, input_srcfile):
    """
    Load an ecRAD output file and merge with inputs.
    Calculate net fluxes and heating rates, including necessary pressures at full levels and half
    levels.
    """

    with xr.open_dataset(output_srcfile) as rad_output, xr.open_dataset(input_srcfile) as ifs_input:

        if 'lat' in ifs_input:
            ifs_input = ifs_input.rename({'lat':'latitude', 'lon':'longitude'})

        input_dset = rad_output.merge(ifs_input) #, compat='override')

        input_dset['pressure_fl'] = xr.DataArray(
                input_dset.pressure_hl[:,:-1] + 0.5*input_dset.pressure_hl.diff('half_level'),
                coords={'column':input_dset.column, 'level':input_dset.level},
                dims=['column', 'level']
            )

        #input_dset['p'] = input_dset.pressure_fl.median('column')
        #input_dset['phl'] = input_dset.pressure_hl.median('column')

        input_dset = input_dset.set_coords(['latitude', 'longitude']) #, 'p', 'phl'])

        input_dset['cloud_radiative_effect_lw'] = (
                (input_dset.flux_dn_lw - input_dset.flux_dn_lw_clear) -\
                (input_dset.flux_up_lw - input_dset.flux_up_lw_clear)
            )
        input_dset['cloud_radiative_effect_sw'] = (
                (input_dset.flux_dn_sw - input_dset.flux_dn_sw_clear) -\
                (input_dset.flux_up_sw - input_dset.flux_up_sw_clear)
            )
        input_dset['flux_net_lw'] = input_dset.flux_dn_lw - input_dset.flux_up_lw
        input_dset['flux_net_sw'] = input_dset.flux_dn_sw - input_dset.flux_up_sw

        input_dset['flux_net_lw_clear'] = input_dset.flux_dn_lw_clear - input_dset.flux_up_lw_clear
        input_dset['flux_net_sw_clear'] = input_dset.flux_dn_sw_clear - input_dset.flux_up_sw_clear

        c_factor = 24*3600*(9.81/1004.)
        input_dset['heating_rate_lw'] = (-1*c_factor*input_dset.flux_net_lw.diff('half_level')\
                /input_dset.pressure_hl.diff('half_level')).rename({'half_level':'level'})
        input_dset['heating_rate_sw'] = (-1*c_factor*input_dset.flux_net_sw.diff('half_level')\
                /input_dset.pressure_hl.diff('half_level')).rename({'half_level':'level'})

        input_dset['heating_rate_lw_clear'] = \
                (-1*c_factor*input_dset.flux_net_lw_clear.diff('half_level')\
                /input_dset.pressure_hl.diff('half_level')).rename({'half_level':'level'})
        input_dset['heating_rate_sw_clear'] = \
                (-1*c_factor*input_dset.flux_net_sw_clear.diff('half_level')\
                /input_dset.pressure_hl.diff('half_level')).rename({'half_level':'level'})

        return input_dset.swap_dims({'column':'latitude'}) # 'half_level':'phl', 'level':'p'})



#!/usr/bin/env python3 

def warn(*args, **kwargs):
    pass
    
import os, warnings
warnings.warn = warn

from ecradplot import plot as eplt

def main(input_srcfile, dstdir):
    """
    Plot input files
    """
    
    if not os.path.isdir(dstdir):
        os.makedirs(dstdir)

    #Get input file name
    name_string      = os.path.splitext(os.path.basename(input_srcfile))[0]
    
    dstfile = os.path.join(dstdir, name_string + ".png")
    
    print(f"Plotting inputs to {dstfile}")
    eplt.plot_inputs(input_srcfile, dstfile=dstfile);
    
if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(description="Plot surface properties, atmospheric composition and clouds from input file to ecRAD.")
    parser.add_argument("input",    help="ecRAD input file")
    parser.add_argument("--dstdir", help="Destination directory for plots", default="./")
    args = parser.parse_args()
    
    main(args.input, args.dstdir)



#!/usr/bin/env python3

def warn(*args, **kwargs):
    pass
    
import os, warnings
warnings.warn = warn

from ecradplot import plot as eplt

def main(input_srcfile, output_srcfile, dstdir):
    """
    Plot radiation fields (fluxes, CRE, and heating rates) 
    """
    
    import os
    if not os.path.isdir(dstdir):
        os.makedirs(dstdir)

    name_string = os.path.splitext(os.path.basename(input_srcfile))[0]
    output_string = os.path.splitext(os.path.basename(output_srcfile))[0]

    dstfile = os.path.join(dstdir, f"{name_string}_{output_string}.png")
    
    print(f"Plotting output to {dstfile}")
    eplt.plot_output(input_srcfile, output_srcfile, dstfile=dstfile);
    
if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(description="Plot radiative fluxes and heating rates from ecRAD output file.")
    parser.add_argument("input",    help="ecRAD input file")
    parser.add_argument("output",   help="ecRAD output file")
    parser.add_argument("--dstdir", help="Destination directory for plots", default="./")
    args = parser.parse_args()
    
    main(args.input, args.output, args.dstdir)



#!/usr/bin/env python
import os
import re
import sys
import fnmatch
import collections

try:
    import argparse
except ImportError:
    import _argparse as argparse

_re_rule = re.compile(
    r'^[ ]*([-+\w./]+(?:[ ]+[-+\w./]+)*)[ ]*'  # targets
    r':(?:[ ]*([-+\w./]+(?:[ ]+[-+\w./]+)*))?[ ]*'  # normal prerequisites
    r'(?:\|[ ]*([-+\w./]+(?:[ ]+[-+\w./]+)*))?')  # order-only prerequisites
_meta_root = 0
_term_colors = {
    'black': 90,
    'red': 91,
    'green': 92,
    'yellow': 93,
    'blue': 94,
    'magenta': 95,
    'cyan': 96,
    'white': 97
}


def parse_args():
    class ArgumentParser(argparse.ArgumentParser):
        def convert_arg_line_to_args(self, arg_line):
            try:
                # Drop everything after the first occurrence of #:
                arg_line = arg_line[:arg_line.index('#')]
            except ValueError:
                pass

            result = []
            # Do not regard consecutive whitespaces as a single separator:
            for arg in arg_line.split(' '):
                if arg:
                    result.append(arg)
                elif result:
                    # The previous argument has a significant space:
                    result[-1] += ' '
            return result

    parser = ArgumentParser(
        fromfile_prefix_chars='@',
        description='Reads a set of MAKEFILEs and prints a topologically '
                    'sorted list of TARGETs (PREREQuisites) together with '
                    'their dependencies (dependents).')

    parser.add_argument(
        '-d', '--debug-file',
        help='dump debug information to DEBUG_FILE')
    parser.add_argument(
        '-t', '--target', nargs='*',
        help='names of the makefile targets to be printed together with their '
             'dependencies; mutually exclusive with the argument '
             '\'-p/--prereq\'; if neither of the arguments is specified, all '
             'targets and prerequisites found in the makefiles are sent to the '
             'output')
    parser.add_argument(
        '-p', '--prereq', nargs='*',
        help='names of the makefile prerequisites to be printed together with '
             'their dependents; mutually exclusive with the argument '
             '\'-t/--target\'; if neither of the arguments is specified, all '
             'targets and prerequisites found in the makefiles are sent to the '
             'output')
    parser.add_argument(
        '--inc-oo', action='store_true',
        help='include order-only prerequisites in the dependency graph')
    parser.add_argument(
        '-r', '--reverse', action='store_true',
        help='print the output list in the reversed order')
    parser.add_argument(
        '--check-unique-prereq', action='append', nargs=2, metavar='PATTERN',
        help='pair of shell-like wildcards; the option enables additional '
             'consistency checks of the dependency graph: each target that '
             'matches the first pattern of the pair is checked whether it has '
             'no more than one prerequisite matching the second pattern; if '
             'the check fails, a warning message is emitted to the standard '
             'error stream')
    parser.add_argument(
        '--check-unique-basename', action='append', nargs='+',
        metavar='PATTERN',
        help='list of shell-like wildcards; the option enables additional '
             'consistency checks of the dependency graph; all targets that '
             'match at least one the patterns are checked whether none of them '
             'have the same basename; if the check fails, a warning message is '
             'emitted to the standard error stream')
    parser.add_argument(
        # Unfortunately, we cannot set nargs to 'two or more', therefore we
        # set nargs to 'one or more':
        '--check-exists-prereq', action='append', nargs='+', metavar='PATTERN',
        help='list of two or more shell-like wildcards; the option enables '
             'additional consistency checks of the dependency graph: each '
             'target that matches the first pattern of the list is checked '
             'whether it has at least one prerequisite matching any of the '
             'rest of the patterns; if the check fails, a warning message is '
             'emitted to the standard error stream')
    parser.add_argument(
        '--check-cycles', action='store_true',
        help='check whether the dependency graph is acyclic, e.g. there is no '
             'circular dependencies; if a cycle is found, a warning message is '
             'emitted to the standard output')
    parser.add_argument(
        '--check-colour', choices=_term_colors.keys(),
        help='colour the message output of the checks using ANSI escape '
             'sequences; the argument is ignored if the standard error stream '
             'is not associated with a terminal device')
    parser.add_argument(
        '-f', '--makefile', nargs='*',
        help='paths to makefiles; a single dash (-) triggers reading from '
             'the standard input stream')

    args = parser.parse_args()

    if args.target is not None and args.prereq is not None:
        parser.error('arguments -t/--target and -p/--prereq are mutually '
                     'exclusive')

    if args.check_exists_prereq:
        for pattern_list in args.check_exists_prereq:
            if len(pattern_list) < 2:
                parser.error('argument --check-exists-prereq: expected 2 or '
                             'more arguments')

    if not sys.stderr.isatty():
        args.check_colour = None

    return args


def read_makefile(makefile, inc_order_only):
    result = collections.defaultdict(list)

    if makefile == '-':
        stream = sys.stdin
    elif not os.path.isfile(makefile):
        return result
    else:
        stream = open(makefile, 'r')

    it = iter(stream)

    for line in it:
        while line.endswith('\\\n'):
            line = line[:-2]
            try:
                line += next(it)
            except StopIteration:
                break

        match = _re_rule.match(line)
        if match:
            targets = set(match.group(1).split())
            prereqs = []

            if match.group(2):
                prereqs.extend(match.group(2).split())

            if match.group(3) and inc_order_only:
                prereqs.extend(match.group(3).split())

            for target in targets:
                result[target].extend(prereqs)

    stream.close()
    return result


def visit_dfs(dep_graph, vertex,
              visited=None,
              start_visit_cb_list=None,
              finish_visit_cb_list=None,
              skip_visit_cb_list=None):
    if visited is None:
        visited = set()

    if vertex in visited:
        if skip_visit_cb_list:
            for skip_visit_cb in skip_visit_cb_list:
                skip_visit_cb(vertex)
        return

    if start_visit_cb_list:
        for start_visit_cb in start_visit_cb_list:
            start_visit_cb(vertex)

    visited.add(vertex)

    if vertex in dep_graph:
        for child in dep_graph[vertex]:
            visit_dfs(dep_graph, child, visited,
                      start_visit_cb_list,
                      finish_visit_cb_list,
                      skip_visit_cb_list)

    if finish_visit_cb_list:
        for finish_visit_cb in finish_visit_cb_list:
            finish_visit_cb(vertex)


def build_graph(makefiles, inc_oo):
    # Read makefiles:
    result = collections.defaultdict(list)
    for mkf in makefiles:
        mkf_dict = read_makefile(mkf, inc_oo)

        for target, prereqs in mkf_dict.items():
            result[target].extend(prereqs)

    # Remove duplicates (we do not use sets as values of the dictionary to keep
    # the order of prerequisites):
    for target in result.keys():
        seen = set()
        result[target] = [prereq for prereq in result[target]
                          if not (prereq in seen or seen.add(prereq))]

    # Make leaves (i.e. prerequisites without any prerequisites) explicit nodes
    # of the graph:
    leaves = set(prereq for prereqs in result.values()
                 for prereq in prereqs if prereq not in result)
    result.update((prereq, result.default_factory()) for prereq in leaves)

    return result


def flip_edges(graph):
    result = collections.defaultdict(list)
    for parent, children in graph.items():
        for child in children:
            result[child].append(parent)
        else:
            _ = result[parent]
    return result


def warn(msg, colour=None):
    sys.stderr.write("%s%s: WARNING: %s%s\n"
                     % (('\033[%dm' % _term_colors[colour]) if colour else '',
                        os.path.basename(__file__),
                        msg,
                        '\033[0m' if colour else ''))


def main():
    args = parse_args()

    if args.debug_file:
        with open(args.debug_file, 'w') as debug_file:
            debug_file.writelines([
                '# Python version: ', sys.version.replace('\n', ' '), '\n',
                '#\n',
                '# Command:\n',
                '#  ', ' '.join(sys.argv), '\n',
                '#\n',
                '# Parsed arguments:\n',
                '#  ', '\n#  '.join(
                    [k + '=' + str(v) for k, v in vars(args).items()]), '\n'])

    if args.makefile is None:
        return

    dep_graph = build_graph(args.makefile, args.inc_oo)

    if not dep_graph:
        return

    if args.prereq is None:
        traversed_graph = dep_graph
        start_nodes = args.target
    else:
        traversed_graph = flip_edges(dep_graph)
        start_nodes = args.prereq

    # Insert _meta_root, which will be the starting-point for the dependency
    # graph traverse:
    if start_nodes is None:
        traversed_graph[_meta_root] = sorted(traversed_graph.keys())
    else:
        traversed_graph[_meta_root] = [t for t in start_nodes
                                       if t in traversed_graph]

    # Visitor callbacks:
    start_visit_cb_list = []
    finish_visit_cb_list = []
    skip_visit_cb_list = []

    # Callbacks that are called once the graph is traversed:
    postprocess_cb_list = []

    if args.check_unique_prereq:
        def check_unique_prereq_start_visit_cb(vertex):
            # Skip if the vertex is _meta_root or does not have descendants:
            if vertex == _meta_root or vertex not in dep_graph:
                return
            for pattern_list in args.check_unique_prereq:
                if fnmatch.fnmatch(vertex, pattern_list[0]):
                    vertex_prereqs = dep_graph[vertex]
                    for prereq_pattern in pattern_list[1:]:
                        matching_prereqs = fnmatch.filter(vertex_prereqs,
                                                          prereq_pattern)
                        if len(matching_prereqs) > 1:
                            warn("target '%s' has more than one immediate "
                                 "prerequisite matching pattern '%s':\n\t%s"
                                 % (vertex,
                                    prereq_pattern,
                                    "\n\t".join(matching_prereqs)),
                                 args.check_colour)

        start_visit_cb_list.append(check_unique_prereq_start_visit_cb)

    if args.check_unique_basename:
        basenames = [collections.defaultdict(set) for _ in
                     range(len(args.check_unique_basename))]

        def check_unique_basename_start_visit_cb(vertex):
            # Skip if the vertex is _meta_root:
            if vertex == _meta_root:
                return
            for i, pattern_list in enumerate(args.check_unique_basename):
                for pattern in pattern_list:
                    if fnmatch.fnmatch(vertex, pattern):
                        basenames[i][os.path.basename(vertex)].add(vertex)

        start_visit_cb_list.append(check_unique_basename_start_visit_cb)

        def check_unique_basename_postprocess_cb():
            for basename_group in basenames:
                for basename, paths in basename_group.items():
                    if len(paths) > 1 and basename:
                        warn("the dependency graph contains more than one "
                             "target with basename '%s':\n\t%s"
                             % (basename, '\n\t'.join(paths)),
                             args.check_colour)

        postprocess_cb_list.append(check_unique_basename_postprocess_cb)

    if args.check_exists_prereq:
        def check_exists_prereq_start_visit_cb(vertex):
            # Skip if the vertex is _meta_root:
            if vertex == _meta_root:
                return
            for pattern_list in args.check_exists_prereq:
                if fnmatch.fnmatch(vertex, pattern_list[0]):
                    vertex_prereqs = dep_graph.get(vertex, set())
                    prereq_patterns = pattern_list[1:]
                    if not any([fnmatch.filter(vertex_prereqs, prereq_pattern)
                                for prereq_pattern in prereq_patterns]):
                        warn("target '%s' does not have an immediate "
                             "prerequisite matching any of the patterns: '%s'"
                             % (vertex,
                                "', '".join(prereq_patterns)),
                             args.check_colour)

        start_visit_cb_list.append(check_exists_prereq_start_visit_cb)

    if args.check_cycles:
        path = []

        def check_cycles_start_visit_cb(vertex):
            path.append(vertex)

        def check_cycles_skip_visit_cb(vertex):
            if vertex in path:
                start_cycle_idx = path.index(vertex)

                if args.prereq is None:
                    msg_lines = (
                            path[1:start_cycle_idx] +
                            [path[start_cycle_idx] + ' <- start of cycle'] +
                            path[start_cycle_idx + 1:] +
                            [vertex + ' <- end of cycle'])
                else:
                    msg_lines = ([vertex + ' <- start of cycle'] +
                                 path[-1:start_cycle_idx:-1] +
                                 [path[start_cycle_idx] + ' <- end of cycle'] +
                                 path[start_cycle_idx - 1:0:-1])

                warn('the dependency graph has a cycle:\n\t%s'
                     % '\n\t'.join(msg_lines), args.check_colour)

        def check_cycles_finish_visit_cb(vertex):
            path.pop()

        start_visit_cb_list.append(check_cycles_start_visit_cb)
        skip_visit_cb_list.append(check_cycles_skip_visit_cb)
        finish_visit_cb_list.append(check_cycles_finish_visit_cb)

    toposort = []

    def toposort_finish_visit_cb(vertex):
        toposort.append(vertex)
    finish_visit_cb_list.append(toposort_finish_visit_cb)

    visit_dfs(traversed_graph, _meta_root,
              start_visit_cb_list=start_visit_cb_list,
              finish_visit_cb_list=finish_visit_cb_list,
              skip_visit_cb_list=skip_visit_cb_list)

    for postprocess_cb in postprocess_cb_list:
        postprocess_cb()

    # The last element of toposort is _meta_root:
    output = '\n'.join(toposort[-2::-1]
                       if (args.reverse ^ (args.prereq is not None))
                       else toposort[:-1])
    if output:
        print(output)


if __name__ == "__main__":
    main()



"""
Filename:     general.py
Author:       Shannon Mason, shannon.mason@ecmwf.int
Description:  Common plotting functions.
"""

import pandas as pd
import numpy as np

#For loading and handling netCDF data
import xarray as xr

def format_time(ax, format_string="%H:%M", label='Time (UTC)'):
    """
    Format axes for time coordinates.
    """
    import matplotlib.dates as mdates
    ax.set_xticklabels(ax.xaxis.get_majorticklabels(), rotation=0, ha='center')
    ax.xaxis.set_major_formatter(mdates.DateFormatter(format_string))
    ax.set_xlabel(label)


def format_height(ax, scale=1.0e3, label='Height [km]'):
    """
    Format axes for height coordinates.
    """
    import matplotlib.ticker as ticker
    ticks_y = ticker.FuncFormatter(lambda x, pos: '${0:g}$'.format(x/scale))
    ax.yaxis.set_major_formatter(ticks_y)
    ax.set_ylabel(label)


def format_temperature(ax, scale='K'):
    """
    Format axes for temperature coordinates.
    """
    import matplotlib.ticker as ticker
    ticks_y = ticker.FuncFormatter(lambda x, pos: '${0:g}$'.format(x))
    ax.yaxis.set_major_formatter(ticks_y)
    ymin, ymax = ax.get_ylim()
    ax.set_ylim(ymax, ymin)
    if scale == 'K':
        ax.set_ylabel('Temperature [K]')
    elif scale == 'C':
        ax.set_ylabel('Temperature [K]')
    else:
        Error("Scale must be either K or C")


def format_pressure(ax, scale=100, label='Pressure [hPa]'):
    """
    Format axes for pressure coordinates.
    """
    import matplotlib.ticker as ticker
    ticks_p = ticker.FuncFormatter(lambda x, pos: '${0:g}$'.format(x/scale))
    ax.yaxis.set_major_formatter(ticks_p)
    ax.set_ylabel(label)


def format_latitude(ax):
    """
    Format axes for latitude coordinates.
    """
    import matplotlib.ticker as ticker
    latFormatter = ticker.FuncFormatter(lambda x, pos: "${:g}^\circ$S".format(-1*x) if x < 0 else "${:g}^\circ$N".format(x))
    ax.xaxis.set_major_formatter(latFormatter)

fancy_format_latitude  = lambda x: r"${:.0f}^{{\circ}}$S".format(-1*x) if x < 0 else "${:.0f}^{{\circ}}$N".format(x)
unfancy_format_latitude  = lambda x: r"{:.0f}S".format(-1*x) if x < 0 else "{:.0f}N".format(x)

def snap_to_axis(ax, ax_ref):
    """
    Align subplot ax with the bounds of subplot ax_ref
    """
    pos_ref = ax_ref.get_position()
    pos = ax.get_position()
    ax.set_position([pos_ref.x0, pos.y0, pos_ref.width, pos.height])

def get_figure_center(ax):
    bbox = ax.get_position()
    return (bbox.get_points()[0][0] + bbox.get_points()[1][0])/2

def get_figure_top(fig, ax, include_hspace=True):
    bbox = ax.get_position()
    if include_hspace:
        return bbox.get_points()[0][1] + fig.subplotpars.hspace
    else:
        return bbox.get_points()[0][1]

def place_suptitle(fig, axes, suptitle, y=0.95, va='top'):
    center = get_figure_center(axes[0])
    fig.suptitle(suptitle, ha='center', x=center, va=va, y=y)

def add_subfigure_labels(axes, xloc=0.0, yloc=1.05, zorder=0, label_list=[], flatten_order='F'):
    if label_list == []:
        import string
        labels = string.ascii_lowercase
    else:
        labels = label_list

    for i, ax in enumerate(axes.flatten(order=flatten_order)):
        ax.text(xloc, yloc, "%s)" %(labels[i]), va='baseline', transform=ax.transAxes, fontweight='bold', zorder=zorder)



#!/usr/bin/env python
import sys

BUF_MAX_SIZE = 512


def _skip_sequence(stream, sequence):
    """
    Finds the first occurrence of a sequence of bytes in a binary stream and
    sets the streams's current position right after it. Returns True if the
    sequence is found and False otherwise, The length of the sequence must not
    exceed BUF_MAX_SIZE.
    """
    sequence_size = len(sequence)
    while 1:
        buf = stream.read(BUF_MAX_SIZE)
        idx = buf.find(sequence)
        if idx < 0:
            if len(buf) < BUF_MAX_SIZE:
                return False
            else:
                stream.seek(1 - sequence_size, 1)
        else:
            stream.seek(idx + sequence_size - len(buf), 1)
            return True


def _mods_differ_default(stream1, stream2):
    # Simple byte comparison:
    while 1:
        buf1 = stream1.read(BUF_MAX_SIZE)
        buf2 = stream2.read(BUF_MAX_SIZE)
        if buf1 != buf2:
            return True
        if not buf1:
            return False


def _mods_differ_intel(stream1, stream2):
    # The first byte encodes the version of the module file format:
    if stream1.read(1) != stream2.read(1):
        return True
    # The block before the following magic sequence might change from
    # compilation to compilation, probably due to a second resolution timestamp
    # in it:
    magic_sequence = b'\x0A\x00'  # the same as \n\0
    if not (_skip_sequence(stream1, magic_sequence) and
            _skip_sequence(stream2, magic_sequence)):
        return True
    return _mods_differ_default(stream1, stream2)


def _mods_differ_gnu(stream1, stream2):
    # The magic number of gzip to be found in the module files generated by
    # GFortran 4.9 or later:
    magic_sequence = b'\x1F\x8B'
    stream1_sequence = stream1.read(len(magic_sequence))
    stream1.seek(0)
    if stream1_sequence != magic_sequence:
        # Older versions of GFortran generate module files in plain ASCII. Also,
        # up to version 4.6.4, the first line of a module file contains a
        # timestamp, therefore we ignore it.
        stream1.readline()
        stream2.readline()
    return _mods_differ_default(stream1, stream2)


def _mods_differ_portland(stream1, stream2):
    for _ in range(2):  # the first two lines must be identical
        if stream1.readline() != stream2.readline():
            return True
    # The next line is a timestamp followed by the sequence '\nenduse\n':
    magic_sequence = b'\x0A\x65\x6E\x64\x75\x73\x65\x0A'
    if not (_skip_sequence(stream1, magic_sequence) and
            _skip_sequence(stream2, magic_sequence)):
        return True
    return _mods_differ_default(stream1, stream2)


def _mods_differ_amd(stream1, stream2):
    # AOCC is based on the Classic Flang, which has the same format as the PGI
    # compiler
    return _mods_differ_portland(stream1, stream2)


def _mods_differ_flang(stream1, stream2):
    # The header of the module files generated by the new Flang compiler,
    # formerly known as F18:
    magic_sequence = (b'\xEF\xBB\xBF\x21'   # UTF-8 BOM
                      b'\x6D\x6F\x64\x24')  # the same as !mod$
    stream1_sequence = stream1.read(len(magic_sequence))
    stream1.seek(0)
    if stream1_sequence != magic_sequence:
        # The Classic Flang has the same format as the PGI compiler
        return _mods_differ_portland(stream1, stream2)
    return _mods_differ_default(stream1, stream2)


def _mods_differ_omni(stream1, stream2):
    import xml.etree.ElementTree as eT
    # Attributes that either declare or reference the type hashes. Each list
    # contains a group of tags that reference "same things".
    hash_attrs = [["imported_id"],
                  ["type", "ref", "return_type", "extends"]]

    tree1 = eT.parse(stream1)
    tree2 = eT.parse(stream2)

    try:
        it1 = tree1.iter()
        it2 = tree2.iter()
    except AttributeError:
        it1 = iter(tree1.getiterator())
        it2 = iter(tree2.getiterator())

    type_maps1 = [dict() for _ in hash_attrs]
    type_maps2 = [dict() for _ in hash_attrs]

    for node1 in it1:
        try:
            node2 = next(it2)
        except StopIteration:
            # The second file is shorter:
            return True

        if node1.tag != node2.tag:
            # The nodes have different tags:
            return True

        if node1.text != node2.text:
            # The nodes have different texts:
            return True

        for ii, attr_group in enumerate(hash_attrs):
            type_map1 = type_maps1[ii]
            type_map2 = type_maps2[ii]

            for attr in attr_group:
                if (attr in node1.attrib) != (attr in node2.attrib):
                    # One of the files has the attribute and the second one
                    # does not:
                    return True

                hash1 = node1.attrib.pop(attr, None)
                hash2 = node2.attrib.pop(attr, None)

                if hash1 == hash2:
                    # Either the attribute is missing in both nodes or they have
                    # the same value:
                    continue
                elif (hash1 in type_map1) != (hash2 in type_map2):
                    # One of the files has already declared the respective hash
                    # and the second one has not:
                    return True
                elif hash1 in type_map1 and \
                        type_map1[hash1] != type_map2[hash2]:
                    # Both files have declared the respective hashes but they
                    # refer to different types:
                    return True
                else:
                    # Declare the respective hashes for both files:
                    type_value = len(type_map1)
                    type_map1[hash1] = type_value
                    type_map2[hash2] = type_value

        if node1.attrib != node2.attrib:
            # The rest of the attributes have different values:
            return True
    try:
        next(it2)
        # The first file is shorter:
        return True
    except StopIteration:
        return False


def mods_differ(filename1, filename2, compiler_name=None):
    """
    Checks whether two Fortran module files are essentially different. Some
    compiler-specific logic is required for compilers that generate different
    module files for the same source file (e.g. the module files might contain
    timestamps). This implementation is inspired by CMake.
    """
    with open(filename1, 'rb') as stream1:
        with open(filename2, 'rb') as stream2:
            if compiler_name == "intel":
                return _mods_differ_intel(stream1, stream2)
            elif compiler_name == "gnu":
                return _mods_differ_gnu(stream1, stream2)
            elif compiler_name == "portland":
                return _mods_differ_portland(stream1, stream2)
            elif compiler_name == "amd":
                return _mods_differ_amd(stream1, stream2)
            elif compiler_name == "flang":
                return _mods_differ_flang(stream1, stream2)
            elif compiler_name == "omni":
                return _mods_differ_omni(stream1, stream2)
            else:
                return _mods_differ_default(stream1, stream2)


# We try to make this as fast as possible, therefore we do not parse arguments
# properly:
exit(mods_differ(sys.argv[1], sys.argv[2],
                 sys.argv[3].lower() if len(sys.argv) > 3 else None))



#!/usr/bin/env python3

def warn(*args, **kwargs):
    pass
    
import os, warnings
warnings.warn = warn

from ecradplot import plot as eplt

def main(input_srcfile, reference_output_srcfile, output_srcfiles, dstdir):
    """
    Plot input files
    """
    
    import os
    if not os.path.isdir(dstdir):
        os.makedirs(dstdir)

    import seaborn as sns
    name_string  = os.path.splitext(os.path.basename(input_srcfile))[0]
    outputs_string = "_".join([os.path.splitext(os.path.basename(f))[0] for f in output_srcfiles])
    reference_string  = os.path.splitext(os.path.basename(reference_output_srcfile))[0]
            
    styles = [{'lw':2, 'color':'k', 'ls':'-', 'zorder':10},
              {'lw':3, 'color':sns.color_palette()[0], 'ls':'-'},
              {'lw':3, 'color':sns.color_palette()[2], 'ls':'-'},
              {'lw':2, 'color':sns.color_palette()[3], 'ls':'--'},
              {'lw':2, 'color':sns.color_palette()[5], 'ls':'-.'},
              {'lw':4, 'color':sns.color_palette()[6], 'ls':'-'},
              {'lw':4, 'color':sns.color_palette()[9], 'ls':'-'}]
        
    reference_name_string = os.path.splitext(os.path.basename(reference_output_srcfile))[0]
    dstfile = f"{dstdir}/{name_string}_{outputs_string}_vs_{reference_name_string}_scalar.png"
    print(f"Plotting integrated, surface and TOA outputs to {dstfile}")
    eplt.compare_output_scalar(input_srcfile, output_srcfiles, reference_output_srcfile, styles, dstfile=dstfile)

    
if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(description="Plot radiative fluxes and heating rates from ecRAD output file.")
    parser.add_argument("input",    help="ecRAD input file")
    parser.add_argument("reference", help="ecRAD output file to use as a reference")
    parser.add_argument("outputs",  help="ecRAD output files", nargs='+')
    parser.add_argument("--dstdir", help="Destination directory for plots", default="./")
    args = parser.parse_args()
        
    main(args.input,  args.reference, args.outputs, args.dstdir)



#!/usr/bin/env python3
import os
import warnings

import seaborn as sns

from ecradplot import plot as eplt

def warn(*args, **kwargs):
    pass

warnings.warn = warn

def format_latitude(latitude):
    if latitude<0:
        return "f{abs(latitude):.0f}S"

    return "f{abs(latitude):.0f}N"

def main(latitude, input_srcfile, reference_output_srcfile, output_srcfiles, dstdir):
    """
    Plot input files
    """

    if not os.path.isdir(dstdir):
        os.makedirs(dstdir)

    name_string  = os.path.splitext(os.path.basename(input_srcfile))[0]
    outputs_string = "_".join([os.path.splitext(os.path.basename(f))[0] for f in output_srcfiles])
    reference_string  = os.path.splitext(os.path.basename(reference_output_srcfile))[0]

    styles = [{'lw':3, 'color':'k', 'ls':'-', 'zorder':10},
              #{'lw':4, 'color':'0.5', 'ls':'-'},
              #{'lw':4, 'color':'0.75', 'ls':'-'},
              {'lw':5, 'color':sns.color_palette()[0], 'ls':'-'},
              {'lw':5, 'color':sns.color_palette()[2], 'ls':'-'},
              {'lw':3, 'color':sns.color_palette()[3], 'ls':'--'},
              {'lw':3, 'color':sns.color_palette()[5], 'ls':'-.'},
              {'lw':5, 'color':sns.color_palette()[6], 'ls':'-'},
              {'lw':5, 'color':sns.color_palette()[9], 'ls':'-'}]

    dstfile = os.path.join(
            dstdir,
            f"{name_string}_{outputs_string}_vs_{reference_string}"
            f"_profile_{format_latitude(latitude)}.png"
        )

    print(f"Plotting output profiles to {dstfile}")
    eplt.compare_output_profile(
            latitude,
            input_srcfile,
            reference_output_srcfile,
            output_srcfiles,
            styles,
            dstfile=dstfile
        )
if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(
            description="Plot radiative fluxes and heating rates from ecRAD output file."
            "If a reference file is given, plot differences with respect to the reference."
        )
    parser.add_argument("latitude",  help="Latitude at which to extract profiles", type=float)
    parser.add_argument("input",     help="ecRAD input file")
    parser.add_argument("reference", help="ecRAD output file to use as a reference", default=None)
    parser.add_argument("outputs",   help="ecRAD output files", nargs='+')
    parser.add_argument("--dstdir",  help="Destination directory for plots", default="./")
    args = parser.parse_args()

    main(args.latitude, args.input, args.reference, args.outputs, args.dstdir)
